==> mpyl-0.0.8.linux-x86_64.tar.gz: github/home/.local/share/virtualenvs/workspace-0SZQOxG8/lib/python3.9/site-packages/mpyl/steps/models.py <==
""" Model representation of run-specific configuration. """

from dataclasses import dataclass
from enum import Enum
from pathlib import Path
from typing import Optional, Dict

from ruamel.yaml import YAML, yaml_object  # type: ignore

from ..project import Project, Stage, Target

yaml = YAML()


@dataclass(frozen=True)
class VersioningProperties:
    revision: str
    branch: Optional[str]
    pr_number: Optional[int]
    tag: Optional[str]

    def __post_init__(self):
        if not self.pr_number and not self.tag:
            raise ValueError('Either pr_number or tag need to be set')

    @property
    def identifier(self):
        return f'pr-{self.pr_number}' if self.pr_number else self.tag


@dataclass(frozen=True)
class RunContext:
    build_id: str
    """Uniquely identifies the run. Typically a monotonically increasing number"""
    run_url: str
    """Link back to the run executor"""
    change_url: str
    """Link to changes"""
    tests_url: str
    """Link to test results"""
    user: str
    """Name of of the user that triggered the run"""
    user_email: Optional[str]
    """Email of of the user that triggered the run"""

    @staticmethod
    def from_configuration(run_details: Dict):
        return RunContext(build_id=run_details['id'], run_url=run_details['run_url'],
                          change_url=run_details['change_url'], tests_url=run_details['tests_url'],
                          user=run_details['user'], user_email=run_details['user_email'])


@yaml_object(yaml)
@dataclass(frozen=True)
class RunProperties:
    """ Contains information that is specific to a particular run of the pipeline"""
    details: RunContext
    """Run specific details"""
    target: Target
    """The deploy target"""
    versioning: VersioningProperties
    config: dict
    """Globally specified configuration, to be used by specific steps. Complies with the schema as
    specified in `mpyl_config.schema.yml`
     """

    @staticmethod
    def from_configuration(run_properties: Dict, config: Dict):
        build = run_properties['build']
        versioning = build['versioning']
        versioning = VersioningProperties(revision=versioning['revision'], branch=versioning['branch'],
                                          pr_number=int(versioning.get('pr_number')), tag=versioning.get('tag'))

        return RunProperties(details=RunContext.from_configuration(build['run']),
                             target=Target(build['parameters']['deploy_target']), versioning=versioning, config=config)


@yaml_object(yaml)
@dataclass(frozen=True)
class ArtifactType(Enum):
    def __eq__(self, other):
        return self.value == other.value

    @classmethod
    def from_yaml(cls, _, node):
        return ArtifactType(int(node.value.split('-')[1]))

    @classmethod
    def to_yaml(cls, representer, node):
        return representer.represent_scalar('!ArtifactType',
                                            f'{node._name_}-{node._value_}')  # pylint: disable=protected-access

    DOCKER_IMAGE = 1
    """A docker image"""
    JUNIT_TESTS = 2
    """A test suite in junit compatible `.xml` format"""
    NONE = 3


@yaml_object(yaml)
@dataclass(frozen=True)
class Artifact:
    artifact_type: ArtifactType
    revision: str
    producing_step: str
    spec: dict


@yaml_object(yaml)
@dataclass(frozen=True)
class Input:
    project: Project
    run_properties: RunProperties
    """Run specific properties"""
    required_artifact: Optional[Artifact] = None
    dry_run: bool = False


@yaml_object(yaml)
@dataclass()
class Output:
    success: bool
    message: str
    produced_artifact: Optional[Artifact] = None

    @staticmethod
    def path(target_path: str, stage: Stage):
        return Path(target_path, f"{stage.name}.yml")

    def write(self, target_path: str, stage: Stage):
        Path(target_path).mkdir(parents=True, exist_ok=True)
        with Output.path(target_path, stage).open(mode='w+', encoding='utf-8') as file:
            yaml.dump(self, file)

    @staticmethod
    def try_read(target_path: str, stage: Stage):
        path = Output.path(target_path, stage)
        if path.exists():
            with open(path, encoding='utf-8') as file:
                return yaml.load(file)
        return None


def input_to_artifact(artifact_type: ArtifactType, step_input: Input, spec: dict):
    return Artifact(artifact_type=artifact_type, revision=step_input.run_properties.versioning.revision,
                    producing_step=step_input.project.name, spec=spec)

==> mpyl-0.0.10.linux-x86_64.tar.gz: github/home/.local/share/virtualenvs/workspace-0SZQOxG8/lib/python3.9/site-packages/mpyl/steps/models.py <==
""" Model representation of run-specific configuration. """

from dataclasses import dataclass
from enum import Enum
from pathlib import Path
from typing import Optional, Dict

from ruamel.yaml import YAML, yaml_object  # type: ignore

from ..project import Project, Stage, Target

yaml = YAML()


@dataclass(frozen=True)
class VersioningProperties:
    revision: str
    branch: Optional[str]
    pr_number: Optional[int]
    tag: Optional[str]

    def __post_init__(self):
        if not self.pr_number and not self.tag:
            raise ValueError('Either pr_number or tag need to be set')

    @property
    def identifier(self):
        return f'pr-{self.pr_number}' if self.pr_number else self.tag


@dataclass(frozen=True)
class RunContext:
    build_id: str
    """Uniquely identifies the run. Typically a monotonically increasing number"""
    run_url: str
    """Link back to the run executor"""
    change_url: str
    """Link to changes"""
    tests_url: str
    """Link to test results"""
    user: str
    """Name of of the user that triggered the run"""
    user_email: Optional[str]
    """Email of of the user that triggered the run"""

    @staticmethod
    def from_configuration(run_details: Dict):
        return RunContext(build_id=run_details['id'], run_url=run_details['run_url'],
                          change_url=run_details['change_url'], tests_url=run_details['tests_url'],
                          user=run_details['user'], user_email=run_details['user_email'])


@yaml_object(yaml)
@dataclass(frozen=True)
class RunProperties:
    """ Contains information that is specific to a particular run of the pipeline"""
    details: RunContext
    """Run specific details"""
    target: Target
    """The deploy target"""
    versioning: VersioningProperties
    config: dict
    """Globally specified configuration, to be used by specific steps. Complies with the schema as
    specified in `mpyl_config.schema.yml`
     """

    @staticmethod
    def for_local_run(config: Dict, revision: str, branch: Optional[str]):
        return RunProperties(details=RunContext("", "", "", "", "", None), target=Target.PULL_REQUEST,
                             versioning=VersioningProperties(revision, branch, 123, None), config=config)

    @staticmethod
    def from_configuration(run_properties: Dict, config: Dict):
        build = run_properties['build']
        versioning = build['versioning']
        versioning = VersioningProperties(revision=versioning['revision'], branch=versioning['branch'],
                                          pr_number=int(versioning.get('pr_number')), tag=versioning.get('tag'))

        return RunProperties(details=RunContext.from_configuration(build['run']),
                             target=Target(build['parameters']['deploy_target']), versioning=versioning, config=config)


@yaml_object(yaml)
@dataclass(frozen=True)
class ArtifactType(Enum):
    def __eq__(self, other):
        return self.value == other.value

    @classmethod
    def from_yaml(cls, _, node):
        return ArtifactType(int(node.value.split('-')[1]))

    @classmethod
    def to_yaml(cls, representer, node):
        return representer.represent_scalar('!ArtifactType',
                                            f'{node._name_}-{node._value_}')  # pylint: disable=protected-access

    DOCKER_IMAGE = 1
    """A docker image"""
    JUNIT_TESTS = 2
    """A test suite in junit compatible `.xml` format"""
    NONE = 3


@yaml_object(yaml)
@dataclass(frozen=True)
class Artifact:
    artifact_type: ArtifactType
    revision: str
    producing_step: str
    spec: dict


@yaml_object(yaml)
@dataclass(frozen=True)
class Input:
    project: Project
    run_properties: RunProperties
    """Run specific properties"""
    required_artifact: Optional[Artifact] = None
    dry_run: bool = False


@yaml_object(yaml)
@dataclass()
class Output:
    success: bool
    message: str
    produced_artifact: Optional[Artifact] = None

    @staticmethod
    def path(target_path: str, stage: Stage):
        return Path(target_path, f"{stage.name}.yml")

    def write(self, target_path: str, stage: Stage):
        Path(target_path).mkdir(parents=True, exist_ok=True)
        with Output.path(target_path, stage).open(mode='w+', encoding='utf-8') as file:
            yaml.dump(self, file)

    @staticmethod
    def try_read(target_path: str, stage: Stage):
        path = Output.path(target_path, stage)
        if path.exists():
            with open(path, encoding='utf-8') as file:
                return yaml.load(file)
        return None


def input_to_artifact(artifact_type: ArtifactType, step_input: Input, spec: dict):
    return Artifact(artifact_type=artifact_type, revision=step_input.run_properties.versioning.revision,
                    producing_step=step_input.project.name, spec=spec)
